          section   TEXT

          xdef      _Q40_test
          xdef      _Q40_mode
          xdef      _Q40_cls
          xdef      _Q40_plot

* Prototype: int Q40_test(void);
* Returns 1 if we are running on a Q40 or 0 if not.

_Q40_test MOVEQ     #0,D0                         | Get system variables
          TRAP      #1                            |  address in A0.
          MOVEQ     #1,D0                         Default flag = TRUE.
          MOVE.B    $A7(A0),D1                    Machine type and display.
          ANDI.B    #$1F,D1                       Wipe out display bits.
          CMPI.B    #$11,D1                       Is it a Q40 ?
          BEQ.S     Q40TEST1                      If yes, jump over...
          MOVEQ     #0,D0                         Flag = FALSE.
Q40TEST1  RTS                                     Return.

* Prototype: void Q40_mode(short mode);
* With:
*         mode = 0 for MODE 8 (256 x 256, 8 colours);
*         mode = 1 for MODE 4 (512 x 256, 4 colours);
*         mode = 2 for 512 x 256, 65536 colours;
*         mode = 3 for 1024 x 512, 65536 colours;

_Q40_mode MOVE.W    4(SP),D0                      Requested mode.
          LEA       $18063,A0                     Emulated mc_stat address.
          LEA       $FF000018,A1                  Display control register addr.
          CMPI.W    #1,D0                         Is it mode 4 or 8 ?
          BGT.S     Q40MODE1                      If not, then jump over...
          CLR.B     (A0)                          Show QL screen.
          MOVEQ     #0,D1                         Default to MODE 4.
          TST.W     D0                            Is it MODE 8 ?
          BNE.S     Q40MODE2                      If not then jump over...
          MOVEQ     #8,D1                         MODE 8.
Q40MODE2  MOVEQ     #-1,D2                        Read display type.
          MOVE.L    A4,-(SP)                      Save context.
          MOVEQ     #$10,D0                       | Set the display mode.
          TRAP      #1                            |
          MOVEA.L   (SP)+,A4                      Restore the context.
          RTS                                     Return.
Q40MODE1  MOVE.B    #2,(A0)                       Blank the QL screen.
          MOVE.B    D0,(A1)                       Set the DCR.
          RTS                                     Return.

* Prototype: void Q40_cls(void);
* Clears the screen memory (all pixels set to black).

_Q40_cls  LEA       $FE800000,A0                  Screen memory address.
          MOVE.W    #$3FFF,D1                     DBRA counter.
          MOVEQ     #0,D0                         Colour = black.
Q40CLS1   MOVE.L    D0,(A0)+                      |
          MOVE.L    D0,(A0)+                      |
          MOVE.L    D0,(A0)+                      |
          MOVE.L    D0,(A0)+                      |
          MOVE.L    D0,(A0)+                      |
          MOVE.L    D0,(A0)+                      |  Clears 64 bytes of
          MOVE.L    D0,(A0)+                      |   memory.
          MOVE.L    D0,(A0)+                      |
          MOVE.L    D0,(A0)+                      |
          MOVE.L    D0,(A0)+                      |
          MOVE.L    D0,(A0)+                      |
          MOVE.L    D0,(A0)+                      |
          MOVE.L    D0,(A0)+                      |
          MOVE.L    D0,(A0)+                      |
          MOVE.L    D0,(A0)+                      |
          MOVE.L    D0,(A0)+                      |
          DBRA      D1,Q40CLS1                    Do it 16384 times...
          RTS                                     Return.

* Prototype: int Q40_plot(short mode, short x, short y, short Green,
*                         short Red, short Blue);

_Q40_plot MOVEM.L   D3-D5,-(SP)                   Save the context.
          LEA       $FE800000,A0                  Screen memory address.
          LEA       16(SP),A1                     Pointer on first parameter.
          MOVE.W    (A1)+,D0                      Display mode.
          MOVE.W    (A1)+,D1                      X coordinate.
          MOVE.W    (A1)+,D2                      Y coordinate.
          CMPI.W    #2,D0                         Is it 512 x 256 mode ?
          BEQ.S     Q40PLOT1                      If yes, then jump over.
          CMPI.W    #512,D2                       Y > 511 ?
          BPL.S     Q40PLOT2                      If yes then return with error.
          MULU      #1024,D2                      Y * line length (in pixels).
          CMPI.W    #1024,D1                      X > 1023 ?
          BMI.S     Q40PLOT3                      If no then jump over...
Q40PLOT2  MOVEQ     #-4,D0                        "Out of range" error code.
          BRA.S     Q40PLOT0                      Restore context and return.
Q40PLOT1  CMPI.W    #256,D2                       Y > 255 ?
          BPL.S     Q40PLOT2                      If yes then return with error.
          MULU      #512,D2                       Y * line length (in pixels).
          CMPI.W    #512,D1                       X > 511 ?
          BPL.S     Q40PLOT2                      If yes then return with error.
Q40PLOT3  MOVEQ     #0,D3                         Clear most significant word.
          MOVE.W    D1,D3                         | Calculate the offset of the
          ADD.L     D2,D3                         |  pixel in the screen
          LSL.L     #1,D3                         |  memory.
          MOVEQ     #0,D4                         Pixel colour.
          MOVEQ     #0,D5                         L.S. bits counter.
          MOVEQ     #2,D0                         DBRA counter (3 colours).
Q40PLOT5  MOVE.W    (A1)+,D1                      Get the colour value.
          LSR.W     #3,D1                         Strip off the three L.S. bits.
          BCC.S     Q40PLOT4                      If bit 2 was 0 then jump...
          ADDQ.B    #1,D5                         Add one to L.S. bit counter.
Q40PLOT4  LSL.W     #5,D4                         Make room for 5 bits.
          OR.W      D1,D4                         Store the colour intensity.
          DBRA      D0,Q40PLOT5                   Get next colour.
          LSL.W     #1,D4                         Make room for intensity bit.
          LSR.W     #1,D5                         Strip off L.S. bit of counter.
          OR.W      D5,D4                         Store the intensity bit.
          MOVE.W    D4,0(A0,D3.L)                 Set the pixel.
          MOVEQ     #0,D0                         No error.
Q40PLOT0  MOVEM.L   (SP)+,D3-D5                   Restore the context.
          RTS

          end
